"""Parser handler."""

import IsoSpecPy as iso
import numpy as np
import pandas as pd
import regex as re
from IsoSpecPy.PeriodicTbl import symbol_to_masses


def get_atom_counts(sequences, modifications, compositions):
    """Get the count occurrence of each atom in a given sequence with respective mods.

    Args:
        sequences (np.array): numpy string array with sequences
        modifications (np.array): numpy string array with modifications
        compositions (dict of dict): dict with each amino acid and modification composition

    Returns:
        elements (list): ordered uniquely occurring elements
        atom_counts (np.array): numpy int array with PSM-level atomic composition
    """
    unique_elements = set()
    for composition in compositions.values():
        for element in composition.keys():
            unique_elements.add(element)

    elements = ["C", "H"] + sorted(unique_elements - set(["C", "H"]))

    atom_counts = np.zeros(shape=(len(sequences), len(elements)), dtype=int)
    for aa_or_mod in compositions.keys():
        ordered_element_multiplier = []
        for element in elements:
            ordered_element_multiplier += [compositions[aa_or_mod].get(element, 0)]
        ordered_element_multiplier = np.array(ordered_element_multiplier)
        # Only amino acids are length 1
        if len(aa_or_mod) == 1:
            atom_counts += np.outer(
                np.char.count(sequences, aa_or_mod), ordered_element_multiplier
            )
        else:
            mod_counts = []
            escaped_mod_name = re.escape(aa_or_mod)
            search_pattern = re.compile(
                rf"(^{escaped_mod_name}:\d+)(?=;)|(?<=;)({escaped_mod_name}:\d+)(?=;)|(?<=;)({escaped_mod_name}:\d+$)|^({escaped_mod_name}:\d+)$"
            )
            for mod in modifications:
                mod_counts.append(
                    len(
                        re.findall(
                            search_pattern,
                            mod,
                        )
                    )
                )
            atom_counts += np.outer(mod_counts, ordered_element_multiplier)
    # Remove water (peptide bonds)
    water = np.zeros(shape=(1, len(elements)), dtype=int)
    water[0, elements.index("H")] = 2
    water[0, elements.index("O")] = 1
    atom_counts -= np.outer(np.maximum(np.char.str_len(sequences) - 1, 0), water)

    return elements, atom_counts


def get_compositions_and_monoisotopic_masses(
    sequences, modifications, compositions, isotopic_distributions
):
    """Get compositions and monoisotopic masses per PSM.

    Args:
        sequences (np.array): numpy string array with sequences
        modifications (np.array): numpy string array with modifications
        compositions (dict of dict): dict with each amino acid and modification composition
        isotopic_distributions (dict of dict): isotopic distributions as provided by chemical_composition

    Returns:
        chemical_compositions (pd.Series): PSM-level chemical compositions in hill notation
        monoisotopic_masses (np.array): numpy float array with PSM-level monoisotopic masses
    """
    # Get atom counts
    elements, atom_counts = get_atom_counts(
        sequences=sequences, modifications=modifications, compositions=compositions
    )
    for i, element in enumerate(elements):
        if i == 0:
            chemical_compositions = np.char.add(
                elements[i] + "(", atom_counts.astype(str)[:, i]
            )
        else:
            next_element_string = np.char.add(
                ")" + elements[i] + "(", atom_counts.astype(str)[:, i]
            )
            chemical_compositions = np.char.add(
                chemical_compositions, next_element_string
            )
        if i == len(elements) - 1:
            chemical_compositions = np.char.add(chemical_compositions, ")")
    # Remove empties
    chemical_compositions = pd.Series(chemical_compositions).str.replace(
        r"\d*[A-Z][a-z]?\(0\)", "", regex=True
    )

    isotope_mass_lookup = {}
    for element, isotope_data in isotopic_distributions.items():
        if isinstance(isotope_data, dict):
            # this has extra data
            isotope_list = []
            for fraction in isotope_data:
                isotope_list.extend(isotope_data[fraction])
        else:
            isotope_list = isotope_data

        for iso_abund in isotope_list:
            isotope_mass_key = f"{round(iso_abund[0])}{element}"
            isotope_mass_lookup[isotope_mass_key] = iso_abund[0]

    monoisotopic_masses = []
    for element in elements:
        try:
            mass = max(isotopic_distributions[element], key=lambda d: d[1])[0]
        except KeyError:
            # Element is an isotope
            mass = isotope_mass_lookup[element]
        monoisotopic_masses.extend([mass])
    monoisotopic_masses = np.array(monoisotopic_masses)
    monoisotopic_masses = (atom_counts * monoisotopic_masses).sum(axis=1)

    return chemical_compositions, monoisotopic_masses


def sort_mods(data):
    """Sort mods based on position in peptide and ensures unique mod:pos pairs.

    Args:
        data (list): list of modifications with style "Mod:position"
    Returns:
        sorted_formatted_mods (str): String with sorted mods in style "Mod1:pos1;Modn:posn"
    """

    def _sort_with_positional_priority(mod):
        if mod == "":
            return None
        else:
            name, _, pos = mod.rpartition(":")
            return int(pos), name

    sorted_formatted_mods = ";".join(
        sorted(set(data), key=_sort_with_positional_priority)
    )
    return sorted_formatted_mods


def init_custom_cc(function, proton):
    """Initialize function for multiprocessing by providing 'global' attribute.

    Args:
        function (function): function to be appended with cc attribute
        proton (float): proton mass
    """

    def _calc_mz(mass, charge):
        return (mass + (charge * proton)) / charge

    function.calc_mz = _calc_mz


def get_isotopologue_accuracy(composition, charge, exp_mz):
    """Compute the isotopologue accuracy.

    Only the accuracy of the isotopologue closest to the experimental mass is reported.

    Args:
        composition (str): chemical composition in hill notation
        charge (int): charge
        exp_mz (float): experimental spectrum mz
    Returns:
        isotopologue_acc (float): accuracy in ppm
    """
    atom_counts = None
    isotope_masses = None
    isotope_probs = None
    replaced_composition = composition
    static_isotopes = re.findall(r"(?<=\))(\d+)(\w+)(?:\()(\d+)", composition)
    if len(static_isotopes) != 0:
        atom_counts = []
        isotope_masses = []
        for isotope in static_isotopes:
            mass, element, number = isotope
            replaced_composition = replaced_composition.replace(
                f"{mass}{element}({number})", ""
            )
            isotope_masses.append(
                [
                    [
                        m
                        for m in symbol_to_masses[element]
                        if str(round(m)).startswith(mass)
                    ][0]
                ]
            )
            atom_counts.append(int(number))
        isotope_probs = len(atom_counts) * [[1.0]]
    if replaced_composition is not None:
        formula = replaced_composition
    else:
        formula = composition
    formula = formula.replace("(", "").replace(")", "")
    isotopologue_masses = list(
        iso.IsoThreshold(
            formula=formula,
            threshold=0.02,
            charge=1,
            get_confs=True,
            atomCounts=atom_counts,
            isotopeMasses=isotope_masses,
            isotopeProbabilities=isotope_probs,
        ).masses
    )
    isotopologue_mzs = [
        get_isotopologue_accuracy.calc_mz(mass, charge) for mass in isotopologue_masses
    ]
    # Report only most accurate mass
    isotopologue_mz = min(
        isotopologue_mzs,
        key=lambda x: abs(exp_mz - x),
    )
    isotopologue_acc = (exp_mz - isotopologue_mz) / isotopologue_mz * 1e6
    return isotopologue_acc



import setuptools


def branch_dependent_version():
    import setuptools_scm

    def void(version):
        return ""

    def version_scheme(version):
        if version.branch not in ["main", "master"]:
            _v = setuptools_scm.get_version(local_scheme=void)
        else:
            _v = str(version.tag)
        return _v

    def local_scheme(version):
        if version.branch not in ["main", "master"]:
            _v = setuptools_scm.get_version(version_scheme=void)
        else:
            _v = ""
        return _v

    scm_version = {
        "root": ".",
        "relative_to": __file__,
        "version_scheme": version_scheme,
        "local_scheme": local_scheme
        # >> "no-local-version" not quit good enough
        # >> on dev "node-and-timestamp",  # version_scheme,
    }
    return scm_version


setuptools.setup(
    use_scm_version=branch_dependent_version,
    setup_requires=["setuptools_scm"],
    include_package_data=True,
)



#!/usr/bin/env python
import pytest

from pyiohat.parsers.quant_base_parser import QuantBaseParser


def test_engine_parsers_QuantBaseParser_init():
    input_file = (
        pytest._test_path / "data" / "test_Creinhardtii_QE_pH11_xtandem_alanine.xml"
    )
    rt_lookup_path = pytest._test_path / "data" / "_ursgal_lookup.csv"
    db_path = pytest._test_path / "data" / "test_Creinhardtii_target_decoy.fasta"

    parser = QuantBaseParser(
        input_file,
        params={
            "cpus": 2,
            "rt_pickle_name": rt_lookup_path,
            "database": db_path,
            "modifications": [
                {
                    "aa": "M",
                    "type": "opt",
                    "position": "any",
                    "name": "Oxidation",
                },
                {
                    "aa": "C",
                    "type": "fix",
                    "position": "any",
                    "name": "Carbamidomethyl",
                },
                {
                    "aa": "*",
                    "type": "opt",
                    "position": "Prot-N-term",
                    "name": "Acetyl",
                },
            ],
        },
    )


def test_engine_parsers_QuantBaseParser_check_parser_compatibility_non_existing():
    # should always return False
    assert QuantBaseParser.check_parser_compatibility("whatever") is False


def test_engine_parsers_QuantBaseParser_check_parser_compatibility_existing():
    # should always return False
    assert (
        QuantBaseParser.check_parser_compatibility(
            pytest._test_path / "data" / "test_Creinhardtii_QE_pH11_xtandem_alanine.xml"
        )
        is False
    )



"""Engine parser."""
import xml.etree.ElementTree as etree

import numpy as np
import pandas as pd
import regex as re
from loguru import logger

from pyiohat.parsers.ident_base_parser import IdentBaseParser


class Comet_2020_01_4_Parser(IdentBaseParser):
    """File parser for Comet."""

    def __init__(self, *args, **kwargs):
        """Initialize parser.

        Reads in data file and provides mappings.
        """
        super().__init__(*args, **kwargs)
        self.version = None
        self.fixed_mods = None
        self.mod_mass_map = None
        self.peptide_lookup = None
        self.spec_records = None
        self.style = "comet_style_1"
        self.mapping_dict = {
            v: k
            for k, v in self.param_mapper.get_default_params(style=self.style)[
                "header_translations"
            ]["translated_value"].items()
        }

    @classmethod
    def check_parser_compatibility(cls, file):
        """Assert compatibility between file and parser.

        Args:
            file (path object): path to input file

        Returns:
            bool: True if parser and file are compatible
        """
        is_mzid = file.name.endswith(".mzid")

        with open(file) as f:
            try:
                head = "".join([next(f) for _ in range(10)])
            except StopIteration:
                head = ""
        contains_engine = "Comet" in head
        return is_mzid and contains_engine

    def get_version(self):
        """Retrieve version from xml.

        Returns:
            version (str): file version
        """
        version = ""
        for event, entry in etree.iterparse(self.input_file):
            entry_tag = entry.tag

            if entry_tag.endswith("cvList"):
                if entry_tag != "{http://psidev.info/psi/pi/mzIdentML/1.2}cvList":
                    logger.warning(
                        f"{entry_tag}: Wrong mzIdentML version - Parser made for version 1.2!"
                    )
            elif entry_tag.endswith("AnalysisSoftware"):
                version = "comet_" + "_".join(
                    re.findall("[0-9]+", entry.attrib["version"])
                )
                break
            entry.clear()
        return version

    def map_mod_mass(self):
        """Retrieve information on modifications from xml.

        Returns:
            fixed_mods (dict): residues and corresponding names of fixed modifications
            mod_mass_map (dict): mapping of masses to modification names
        """
        fixed_mods = {}
        mod_mass_map = {}

        mod_name = ""
        modification_information = False

        for event, entry in etree.iterparse(self.input_file):
            entry_tag = entry.tag

            if entry_tag.endswith("AdditionalSearchParams"):
                modification_information = True
            elif modification_information is True:
                if entry_tag.endswith("cvParam"):
                    mod_name = entry.attrib["name"]
                elif entry_tag.endswith("SearchModification"):
                    if mod_name == "unknown modification":
                        potential_mod = self.mod_mapper.mass_to_names(
                            float(entry.attrib["massDelta"]), decimals=4
                        )
                        if len(potential_mod) == 0:
                            logger.error(
                                f"Cannot map modification with mass {entry.attrib['massDelta']}."
                            )
                            raise ValueError
                        else:
                            mod_name = potential_mod[0]
                    mod_mass_map[entry.attrib["massDelta"]] = mod_name
                    if entry.attrib["fixedMod"] == "true":
                        residue = entry.attrib["residues"]
                        fixed_mods[residue] = mod_name
                elif entry_tag.endswith("ModificationParams"):
                    break
            entry.clear()
        return fixed_mods, mod_mass_map

    def get_peptide_lookup(self):
        """Retrieve peptide ids with their corresponding sequences and modifications from xml.

        Returns:
            peptide_lookup (dict): peptide id with corresponding modifications and sequence
        """
        peptide_lookup = {}

        modifications = []
        sequence = {}
        peptide_information = False

        for event, entry in etree.iterparse(self.input_file):
            entry_tag = entry.tag

            if entry_tag.endswith("PeptideSequence"):
                peptide_information = True
            if peptide_information is True:
                if entry_tag.endswith("PeptideSequence"):
                    sequence = entry.text
                    if len(self.fixed_mods) > 0:
                        sequence_mod_map = self.map_mods_sequences(sequence)
                        if sequence_mod_map != "":
                            modifications.append(sequence_mod_map)
                elif entry_tag.endswith("Modification"):
                    mass = entry.attrib["monoisotopicMassDelta"]
                    location = entry.attrib["location"]
                    mass_name = self.mod_mass_map[mass]
                    modifications.append(mass_name + ":" + location)
                elif entry_tag.endswith("Peptide"):
                    peptide_lookup[entry.attrib["id"]] = (
                        ";".join(modifications),
                        sequence,
                    )
                    modifications = []
                elif entry_tag.endswith("PeptideEvidence"):
                    break
            entry.clear()
        return peptide_lookup

    def get_spec_records(self):
        """Retrieve specs from file.

        Returns:
            spec_records (list): information on PSMs
        """
        spec_records = []

        spec_results = {}
        spec_ident_items = []
        spec_information = False

        for event, entry in etree.iterparse(self.input_file):
            entry_tag = entry.tag

            if entry_tag.endswith("PeptideEvidenceRef"):
                spec_information = True
            if spec_information is True:
                if entry_tag.endswith("cvParam"):
                    if entry.attrib["name"] in self.mapping_dict:
                        _key = self.mapping_dict[entry.attrib["name"]]
                        spec_results[_key] = entry.attrib["value"]
                elif entry_tag.endswith("SpectrumIdentificationItem"):
                    for attribute in list(entry.attrib):
                        if attribute in self.mapping_dict.keys():
                            _key = self.mapping_dict[attribute]
                            spec_results[_key] = entry.attrib[attribute]
                    mods, sequence = self.peptide_lookup[spec_results["sequence"]]
                    spec_results["modifications"] = mods
                    spec_results["sequence"] = sequence
                    spec_ident_items.append(spec_results)
                    spec_results = {}
                elif entry_tag.endswith("SpectrumIdentificationResult"):
                    for spec_item in spec_ident_items:
                        spec_item.update(spec_results)
                        spec_item["spectrum_id"] = entry.attrib["spectrumID"].lstrip(
                            "scan="
                        )
                        spec_records.append(spec_item)
                    spec_results = {}
                    spec_ident_items = []
                elif entry_tag.endswith("SpectrumIdentificationList"):
                    break
            entry.clear()
        return spec_records

    def map_mods_sequences(self, sequence):
        """Map fixed_mods and sequence to get corresponding modifications.

        fixed_mods needs at least one entry!

        Args:
            sequence (str): peptide sequence

        Returns:
            fixed_mod_strings (str): modifications of corresponding sequence
        """
        fixed_mod_strings = []

        for fm_res, fm_name in self.fixed_mods.items():
            fixed_mod_strings.append(
                pd.Series(sequence)
                .str.split(fm_res)
                .apply(
                    lambda l: ";".join(
                        [
                            fm_name + ":" + ind
                            for ind in (
                                np.cumsum(list(map(len, l[:-1]))) + range(1, len(l))
                            ).astype(str)
                        ]
                    )
                )
            )

        fixed_mod_strings = (
            pd.concat(fixed_mod_strings, axis=1).agg(";".join, axis=1).str.rstrip(";")
        )
        fixed_mod_strings = fixed_mod_strings[0]
        return fixed_mod_strings

    def unify(self):
        """
        Primary method to read and unify engine output.

        Returns:
            self.df (pd.DataFrame): unified dataframe
        """
        self.version = self.get_version()
        self.fixed_mods, self.mod_mass_map = self.map_mod_mass()
        self.peptide_lookup = self.get_peptide_lookup()
        self.spec_records = self.get_spec_records()
        self.df = pd.DataFrame(self.spec_records)
        self.df["search_engine"] = self.version
        self.process_unify_style()

        return self.df



import pytest

from pyiohat.parsers.ident.msfragger_3_parser import MSFragger_3_Parser


def test_engine_parsers_msfragger_init():
    input_file = (
        pytest._test_path / "data" / "test_Creinhardtii_QE_pH11_msfragger_3.tsv"
    )
    parser = MSFragger_3_Parser(
        input_file,
        params={
            "cpus": 2,
            "enzyme": "(?<=[KR])(?![P])",
            "terminal_cleavage_site_integrity": "any",
            "validation_score_field": {"msfragger_3_0": "msfragger:hyperscore"},
            "bigger_scores_better": {"msfragger_3_0": True},
            "modifications": [
                {
                    "aa": "M",
                    "type": "opt",
                    "position": "any",
                    "name": "Oxidation",
                },
                {
                    "aa": "C",
                    "type": "fix",
                    "position": "any",
                    "name": "Carbamidomethyl",
                },
                {
                    "aa": "*",
                    "type": "opt",
                    "position": "Prot-N-term",
                    "name": "Acetyl",
                },
            ],
        },
    )


def test_engine_parsers_msfragger_check_parser_compatibility():
    input_file = (
        pytest._test_path / "data" / "test_Creinhardtii_QE_pH11_msfragger_3.tsv"
    )
    assert MSFragger_3_Parser.check_parser_compatibility(input_file) is True


def test_engine_parsers_msfragger_check_dataframe_integrity():
    input_file = pytest._test_path / "data" / "BSA1_msfragger_3.tsv"
    rt_lookup_path = pytest._test_path / "data" / "BSA1_ursgal_lookup.csv"
    db_path = pytest._test_path / "data" / "BSA.fasta"

    parser = MSFragger_3_Parser(
        input_file,
        params={
            "cpus": 2,
            "rt_pickle_name": rt_lookup_path,
            "database": db_path,
            "enzyme": "(?<=[KR])(?![P])",
            "terminal_cleavage_site_integrity": "any",
            "validation_score_field": {"msfragger_3_0": "msfragger:hyperscore"},
            "bigger_scores_better": {"msfragger_3_0": True},
            "modifications": [
                {
                    "aa": "M",
                    "type": "opt",
                    "position": "any",
                    "name": "Oxidation",
                },
                {
                    "aa": "C",
                    "type": "fix",
                    "position": "any",
                    "name": "Carbamidomethyl",
                },
                {
                    "aa": "*",
                    "type": "opt",
                    "position": "Prot-N-term",
                    "name": "Acetyl",
                },
            ],
            "15N": False,
        },
    )
    df = parser.unify()
    assert len(df) == 3417
    assert pytest.approx(df["ucalc_mz"].mean()) == 477.8585
    assert pytest.approx(df["exp_mz"].mean()) == 478.12137

    assert df["modifications"].str.contains("Acetyl:0").sum() == 2
    assert df["modifications"].str.contains("Oxidation:").sum() == 221
    assert (
        df["modifications"].str.count("Carbamidomethyl:")
        == df["sequence"].str.count("C")
    ).all()
    assert df["modifications"].str.count(":").sum() == 2242
    assert (df["raw_data_location"] == "path/for/glory.mzML").all()


def test_map_mod_translation_msfragger3():
    input_file = (
        pytest._test_path / "data" / "test_Creinhardtii_QE_pH11_msfragger_3.tsv"
    )

    parser = MSFragger_3_Parser(
        input_file,
        params={
            "cpus": 2,
            "enzyme": "(?<=[KR])(?![P])",
            "terminal_cleavage_site_integrity": "any",
            "validation_score_field": {"msfragger_3_0": "msfragger:hyperscore"},
            "bigger_scores_better": {"msfragger_3_0": True},
            "modifications": [
                {
                    "aa": "M",
                    "type": "opt",
                    "position": "any",
                    "name": "Oxidation",
                },
                {
                    "aa": "C",
                    "type": "fix",
                    "position": "any",
                    "name": "Carbamidomethyl",
                },
                {
                    "aa": "*",
                    "type": "opt",
                    "position": "Prot-N-term",
                    "name": "Acetyl",
                },
            ],
            "15N": False,
        },
    )
    map_dict = {"15.994915": ["Oxidation"], "57.021465": ["Carbamidomethyl"]}
    converted = parser._map_mod_translation(
        row=["3M(15.994915)", "15M(15.994915)", "18C(57.021465)"], map_dict=map_dict
    )
    assert converted == "Oxidation:3;Oxidation:15;Carbamidomethyl:18;"


def test_c_terminal_tmt():
    input_file = pytest._test_path / "data" / "test_positions_msfragger.tsv"

    parser = MSFragger_3_Parser(
        input_file,
        params={
            "cpus": 2,
            "enzyme": "(?<=[KR])(?![P])",
            "terminal_cleavage_site_integrity": "any",
            "validation_score_field": {"msfragger_3_0": "msfragger:hyperscore"},
            "bigger_scores_better": {"msfragger_3_0": True},
            "modifications": [
                {
                    "aa": "M",
                    "type": "opt",
                    "position": "any",
                    "name": "Oxidation",
                },
                {
                    "aa": "*",
                    "type": "opt",
                    "position": "Prot-N-term",
                    "name": "Acetyl",
                },
                {
                    "aa": "*",
                    "type": "opt",
                    "position": "N-term",
                    "name": "TMT6plex",
                },
                {
                    "aa": "C",
                    "type": "fix",
                    "position": "any",
                    "name": "Carbamidomethyl",
                },
                {
                    "aa": "K",
                    "type": "fix",
                    "position": "any",
                    "name": "TMT6plex",
                },
            ],
            "15N": False,
        },
    )
    converted = parser.translate_mods()
    assert converted[0] == "TMT6plex:0;TMT6plex:6"


def test_msfragger_open_search():
    input_file = pytest._test_path / "data" / "BSA1_open_search.msfragger.tsv"
    rt_lookup_path = pytest._test_path / "data" / "BSA1_ursgal_lookup.csv"
    db_path = pytest._test_path / "data" / "BSA.fasta"

    parser = MSFragger_3_Parser(
        input_file,
        params={
            "cpus": 2,
            "rt_pickle_name": rt_lookup_path,
            "database": db_path,
            "enzyme": "(?<=[KR])(?![P])",
            "terminal_cleavage_site_integrity": "any",
            "validation_score_field": {"msfragger_3_0": "msfragger:hyperscore"},
            "bigger_scores_better": {"msfragger_3_0": True},
            "modifications": [],
            "15N": False,
        },
    )
    df = parser.unify()
    assert df["mass_delta"].mean() == pytest.approx(458.901, abs=1e-6)



"""Engine parser."""
import pandas as pd
import regex as re

from pyiohat.parsers.ident_base_parser import IdentBaseParser


class MSAmanda_2_Parser(IdentBaseParser):
    """File parser for MS Amanda 2."""

    def __init__(self, *args, **kwargs):
        """Initialize parser.

        Reads in data file and provides mappings.
        """
        super().__init__(*args, **kwargs)
        self.style = "msamanda_style_1"

        self.df = pd.read_csv(self.input_file, delimiter="\t", skiprows=1)
        self.df.dropna(axis=1, how="all", inplace=True)

        self.mapping_dict = {
            v: k
            for k, v in self.param_mapper.get_default_params(style=self.style)[
                "header_translations"
            ]["translated_value"].items()
        }
        self.df.rename(columns=self.mapping_dict, inplace=True)
        self.df.columns = self.df.columns.str.lstrip(" ")
        if not "modifications" in self.df.columns:
            self.df["modifications"] = ""

        self.df.drop(
            columns=[
                c
                for c in self.df.columns
                if c
                not in set(self.mapping_dict.values()) | set(self.reference_dict.keys())
            ],
            inplace=True,
            errors="ignore",
        )
        self.reference_dict.update({k: None for k in self.mapping_dict.values()})

    @classmethod
    def check_parser_compatibility(cls, file):
        """Assert compatibility between file and parser.

        Args:
            file (str): path to input file

        Returns:
            bool: True if parser and file are compatible

        """
        # It is a csv file even though it is technically tab-delimited
        is_csv = file.as_posix().endswith(".csv")
        with open(file.as_posix()) as f:
            try:
                head = "".join([next(f) for _ in range(1)])
            except StopIteration:
                head = ""
        matches_version = "#version: 2." in head
        return is_csv and matches_version

    def _map_mod_translation(self, row):
        """Replace single mod string.

        Args:
            row (str): unprocessed modification string

        Returns:
            mod_str (str): formatted modification string
        """
        mod_str = ""
        if row == "" or row == [""]:
            return mod_str
        for mod in row:
            mod_name = re.search(r"\(([^|]+)", mod).group(1)
            pos = mod.split("(")[0]
            if "N-TERM" in pos.upper():
                pos = 0
            else:
                pos = int(re.search(r"\d+", mod).group(0))
            mod_str += f"{mod_name}:{pos};"
        return mod_str

    def translate_mods(self):
        """
        Replace internal modification nomenclature with formatted modification strings.

        Returns:
            (pd.Series): column with formatted mod strings
        """
        mod_split_col = self.df["modifications"].fillna("").str.split(";")
        mods_translated = mod_split_col.apply(self._map_mod_translation)

        return mods_translated.str.rstrip(";")

    def unify(self):
        """
        Primary method to read and unify engine output.

        Returns:
            self.df (pd.DataFrame): unified dataframe
        """
        self.df["search_engine"] = "msamanda_2_0_0_17442"
        self.df["spectrum_id"] = self.df["spectrum_title"].str.split(".").str[-3]
        self.df["modifications"] = self.translate_mods()
        self.process_unify_style()

        return self.df
